Class: Hanami::Http::Status Private

Inherits:
Object
  • Object
show all
Defined in:
lib/hanami/http/status.rb

Overview

This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.

An HTTP status

Since:

  • 0.1.0

Constant Summary collapse

ALL =

This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.

A set of standard codes and messages for HTTP statuses

Since:

  • 0.1.0

::Rack::Utils::HTTP_STATUS_CODES

Class Method Summary collapse

Class Method Details

.for_code(code) ⇒ Array

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Return a status for the given code

Examples:

require 'hanami/http/status'

Hanami::Http::Status.for_code(418) # => [418, "I'm a teapot"]

Parameters:

  • code (Integer)

    a valid HTTP code

Returns:

  • (Array)

    a pair of code and message for an HTTP status

Since:

  • 0.1.0



33
34
35
# File 'lib/hanami/http/status.rb', line 33

def self.for_code(code)
  ALL.assoc(code)
end

.message_for(code) ⇒ String

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Return a message for the given status code

Parameters:

  • code (Integer)

    a valid HTTP code

Returns:

  • (String)

    a message for the given status code

Since:

  • 0.3.2



45
46
47
# File 'lib/hanami/http/status.rb', line 45

def self.message_for(code)
  for_code(code)[1]
end