Class: Hanami::Action::Response Private

Inherits:
Rack::Response
  • Object
show all
Defined in:
lib/hanami/action/response.rb

Overview

This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.

The HTTP response for an action, given to #handle.

Inherits from Rack::Response, providing compatibility with Rack functionality.

Constant Summary collapse

DEFAULT_VIEW_OPTIONS =

This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0

-> (*) { {} }.freeze
EMPTY_BODY =

This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0

[].freeze
FILE_SYSTEM_ROOT =

This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0

Pathname.new("/").freeze

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(request:, config:, content_type: nil, env: {}, headers: {}, view_options: nil, sessions_enabled: false) ⇒ Response

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns a new instance of Response.

Since:

  • 2.0.0



51
52
53
54
55
56
57
58
59
60
61
62
63
64
# File 'lib/hanami/action/response.rb', line 51

def initialize(request:, config:, content_type: nil, env: {}, headers: {}, view_options: nil, sessions_enabled: false) # rubocop:disable Layout/LineLength, Metrics/ParameterLists
  super([], 200, headers.dup)
  self.content_type = content_type if content_type

  @request = request
  @config = config
  @charset = ::Rack::MediaType.params(content_type).fetch("charset", nil)
  @exposures = {}
  @env = env
  @view_options = view_options || DEFAULT_VIEW_OPTIONS

  @sessions_enabled = sessions_enabled
  @sending_file = false
end

Instance Attribute Details

#charsetObject

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



37
38
39
# File 'lib/hanami/action/response.rb', line 37

def charset
  @charset
end

#envObject (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



33
34
35
# File 'lib/hanami/action/response.rb', line 33

def env
  @env
end

#exposuresObject (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



33
34
35
# File 'lib/hanami/action/response.rb', line 33

def exposures
  @exposures
end

#requestObject (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



33
34
35
# File 'lib/hanami/action/response.rb', line 33

def request
  @request
end

#view_optionsObject (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



33
34
35
# File 'lib/hanami/action/response.rb', line 33

def view_options
  @view_options
end

Class Method Details

.build(status, env) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



41
42
43
44
45
46
47
# File 'lib/hanami/action/response.rb', line 41

def self.build(status, env)
  new(config: Action.config.dup, content_type: Mime.best_q_match(env[Action::HTTP_ACCEPT]), env: env).tap do |r|
    r.status = status
    r.body   = Http::Status.message_for(status)
    r.set_format(Mime.detect_format(r.content_type), config)
  end
end

Instance Method Details

#[](key) ⇒ Object

Returns the exposure value for the given key.

Parameters:

  • key (Object)

Returns:

  • (Object)

    the exposure value, if found

Raises:

  • (KeyError)

    if the exposure was not found

Since:

  • 2.0.0



152
153
154
# File 'lib/hanami/action/response.rb', line 152

def [](key)
  @exposures.fetch(key)
end

#[]=(key, value) ⇒ Object

Sets an exposure value for the given key.

Parameters:

  • key (Object)
  • value (Object)

Returns:

  • (Object)

    the value

Since:

  • 2.0.0



165
166
167
# File 'lib/hanami/action/response.rb', line 165

def []=(key, value)
  @exposures[key] = value
end

#_send_file(send_file_response) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



412
413
414
415
416
417
418
419
420
421
422
423
424
# File 'lib/hanami/action/response.rb', line 412

def _send_file(send_file_response)
  headers.merge!(send_file_response[Action::RESPONSE_HEADERS])

  if send_file_response[Action::RESPONSE_CODE] == Action::NOT_FOUND
    headers.delete(Action::X_CASCADE)
    headers.delete(Action::CONTENT_LENGTH)
    Halt.call(Action::NOT_FOUND)
  else
    self.status = send_file_response[Action::RESPONSE_CODE]
    self.body = send_file_response[Action::RESPONSE_BODY]
    @sending_file = true
  end
end

#allow_redirect?Boolean

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns:

  • (Boolean)

Since:

  • 2.0.0



394
395
396
397
398
# File 'lib/hanami/action/response.rb', line 394

def allow_redirect?
  return body.empty? if body.respond_to?(:empty?)

  !@sending_file
end

#body=(str) ⇒ Object

Sets the response body.

Parameters:

  • str (String)

    the body string

Since:

  • 2.0.0



72
73
74
75
76
77
78
79
80
81
82
# File 'lib/hanami/action/response.rb', line 72

def body=(str)
  @length = 0
  @body   = EMPTY_BODY.dup

  # FIXME: there could be a bug that prevents Content-Length to be sent for files
  if str.is_a?(::Rack::File::Iterator)
    @body = str
  else
    write(str) unless str.nil? || str == EMPTY_BODY
  end
end

#cache_control(*values) ⇒ Object

Specifies the response freshness policy for HTTP caches using the Cache-Control header.

Any number of non-value directives (:public, :private, :no_cache, :no_store, :must_revalidate, :proxy_revalidate) may be passed along with a Hash of value directives (:max_age, :min_stale, :s_max_age).

See [RFC 2616 / 14.9](tools.ietf.org/html/rfc2616#section-14.9.1) for more on standard cache control directives.

Examples:

# Set Cache-Control directives
response.cache_control :public, max_age: 900, s_maxage: 86400

# Overwrite previous Cache-Control directives
response.cache_control :private, :no_cache, :no_store

response.get_header("Cache-Control") # => "private, no-store, max-age=900"

Parameters:

  • values (Array<Symbol, Hash>)

    values to map to Cache-Control directives

Options Hash (*values):

  • :public (Symbol)
  • :private (Symbol)
  • :no_cache (Symbol)
  • :no_store (Symbol)
  • :must_validate (Symbol)
  • :proxy_revalidate (Symbol)
  • :max_age (Hash)
  • :min_stale (Hash)
  • :s_max_age (Hash)

Returns:

  • void

Since:

  • 2.0.0



312
313
314
315
# File 'lib/hanami/action/response.rb', line 312

def cache_control(*values)
  directives = Cache::CacheControl::Directives.new(*values)
  headers.merge!(directives.headers)
end

#cookiesCookieJar

Returns the set of cookies to be included in the response.

Returns:

Since:

  • 2.0.0



215
216
217
# File 'lib/hanami/action/response.rb', line 215

def cookies
  @cookies ||= CookieJar.new(env.dup, headers, @config.cookies)
end

#expires(amount, *values) ⇒ Object

Sets the Expires header and Cache-Control/max-age directive for the response.

You can provide an integer number of seconds in the future, or a Time object indicating when the response should be considered “stale”. The remaining arguments are passed to #cache_control.

Examples:

# Set Cache-Control directives and Expires
response.expires 900, :public

# Overwrite Cache-Control directives and Expires
response.expires 300, :private, :no_cache, :no_store

response.get_header("Expires") # => "Thu, 26 Jun 2014 12:00:00 GMT"
response.get_header("Cache-Control") # => "private, no-cache, no-store max-age=300"

Parameters:

  • amount (Integer, Time)

    number of seconds or point in time

  • values (Array<Symbols>)

    values to map to Cache-Control directives via #cache_control

Returns:

  • void

Since:

  • 2.0.0



341
342
343
344
# File 'lib/hanami/action/response.rb', line 341

def expires(amount, *values)
  directives = Cache::Expires::Directives.new(amount, *values)
  headers.merge!(directives.headers)
end

#flashFlash

Returns the flash for the request.

This is the same flash object as the Hanami::Action::Request.

Returns:

Raises:

See Also:

Since:

  • 2.0.0



201
202
203
204
205
206
207
# File 'lib/hanami/action/response.rb', line 201

def flash
  unless @sessions_enabled
    raise Hanami::Action::MissingSessionError.new("Hanami::Action::Response#flash")
  end

  request.flash
end

#formatSymbol?

Returns the format for the response.

Returns nil if a format has not been assigned and also cannot be determined from the response’s #content_type.

Examples:

response.format # => :json

Returns:

  • (Symbol, nil)

Since:

  • 2.0.0



103
104
105
# File 'lib/hanami/action/response.rb', line 103

def format
  @format ||= Mime.detect_format(content_type, @config)
end

#format=(value) ⇒ Object

Sets the format and associated content type for the response.

Either a format name (:json) or a MIME type (‘“application/json”`) may be given. In either case, the format or content type will be derived from the given value, and both will be set.

Providing an unknown format name will raise an UnknownFormatError.

Providing an unknown MIME type will set the content type and set the format as nil.

Examples:

Assigning via a format name symbol

response.format = :json
response.content_type # => "application/json"
response.headers["Content-Type"] # => "application/json"

Assigning via a content type string

response.format = "application/json"
response.format # => :json
response.content_type # => "application/json"

Parameters:

  • value (Symbol, String)

    the format name or content type

Raises:

See Also:

Since:

  • 2.0.0



134
135
136
137
138
139
140
# File 'lib/hanami/action/response.rb', line 134

def format=(value)
  format, content_type = Mime.detect_format_and_content_type(value, @config)

  self.content_type = Mime.content_type_with_charset(content_type, charset)

  @format = format
end

#fresh(options) ⇒ Object

Sets the etag and/or last_modified headers on the response and halts with a ‘304 Not Modified` response if the request is still fresh according to the IfNoneMatch and IfModifiedSince request headers.

Examples:

# Set etag header and halt 304 if request matches IF_NONE_MATCH header
response.fresh etag: some_resource.updated_at.to_i

# Set last_modified header and halt 304 if request matches IF_MODIFIED_SINCE
response.fresh last_modified: some_resource.updated_at

# Set etag and last_modified header and halt 304 if request matches IF_MODIFIED_SINCE and IF_NONE_MATCH
response.fresh last_modified: some_resource.updated_at

Parameters:

  • options (Hash)

Options Hash (options):

  • :etag (Integer)

    for testing IfNoneMatch conditions

  • :last_modified (Date)

    for testing IfModifiedSince conditions

Returns:

  • void

Since:

  • 2.0.0



368
369
370
371
372
373
374
375
376
# File 'lib/hanami/action/response.rb', line 368

def fresh(options)
  conditional_get = Cache::ConditionalGet.new(env, options)

  headers.merge!(conditional_get.headers)

  conditional_get.fresh? do
    Halt.call(304)
  end
end

#head?Boolean

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns:

  • (Boolean)

Since:

  • 2.0.0



406
407
408
# File 'lib/hanami/action/response.rb', line 406

def head?
  env[Action::REQUEST_METHOD] == Action::HEAD
end

#redirect_to(url, status: 302) ⇒ Object

Sets the response to redirect to the given URL and halts further handling.

Parameters:

  • url (String)
  • status (Integer) (defaults to: 302)

    the HTTP status to use for the redirect

Since:

  • 2.0.0



226
227
228
229
230
231
# File 'lib/hanami/action/response.rb', line 226

def redirect_to(url, status: 302)
  return unless allow_redirect?

  redirect(::String.new(url), status)
  Halt.call(status)
end

#render(view, **options) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

This is NOT RELEASED with 2.0.0

Since:

  • 2.0.0



87
88
89
# File 'lib/hanami/action/response.rb', line 87

def render(view, **options)
  self.body = view.(**view_options.(request, self), **exposures.merge(options)).to_str
end

#renderable?Boolean

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns:

  • (Boolean)

Since:

  • 2.0.0



386
387
388
389
390
# File 'lib/hanami/action/response.rb', line 386

def renderable?
  return !head? && body.empty? if body.respond_to?(:empty?)

  !@sending_file && !head?
end

#send_file(path) ⇒ void

This method returns an undefined value.

Sends the file at the given path as the response, for any file within the configured public_directory.

Handles the following aspects for file responses:

  • Setting Content-Type and Content-Length headers

  • File Not Found responses (returns a 404)

  • Conditional GET (via If-Modified-Since header)

  • Range requests (via Range header)

Parameters:

  • path (String)

    the file path

See Also:

Since:

  • 2.0.0



251
252
253
254
255
# File 'lib/hanami/action/response.rb', line 251

def send_file(path)
  _send_file(
    Rack::File.new(path, @config.public_directory).call(env)
  )
end

#sessionHash

Returns the session for the response.

This is the same session object as the Hanami::Action::Request.

Returns:

  • (Hash)

    the session object

Raises:

See Also:

Since:

  • 2.0.0



181
182
183
184
185
186
187
# File 'lib/hanami/action/response.rb', line 181

def session
  unless @sessions_enabled
    raise Hanami::Action::MissingSessionError.new("Hanami::Action::Response#session")
  end

  request.session
end

#set_format(value) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Since:

  • 2.0.0



380
381
382
# File 'lib/hanami/action/response.rb', line 380

def set_format(value) # rubocop:disable Naming/AccessorMethodName
  @format = value
end

#unsafe_send_file(path) ⇒ void

This method returns an undefined value.

Send the file at the given path as the response, for a file anywhere in the file system.

Parameters:

  • path (String, Pathname)

    path to the file to be sent

See Also:

Since:

  • 2.0.0



267
268
269
270
271
272
273
274
275
276
277
# File 'lib/hanami/action/response.rb', line 267

def unsafe_send_file(path)
  directory = if Pathname.new(path).relative?
                @config.root_directory
              else
                FILE_SYSTEM_ROOT
              end

  _send_file(
    Rack::File.new(path, directory).call(env)
  )
end